@@ -20,11 +20,16 @@ statsbomb_warning = ('Please be responsible with Statsbomb data.'
 
 def _split_location_cols(df, col, new_cols):
     """ Location is stored as a list. split into columns. """
+    for new_col in new_cols:
+        df[new_col] = np.nan
     if col in df.columns:
-        df[new_cols] = df[col].apply(pd.Series)
+        mask_not_null = df[col].notnull()
+        df_not_null = df.loc[mask_not_null, col]
+        df_new = pd.DataFrame(df_not_null.tolist(), index=df_not_null.index, columns=new_cols)
+        df.loc[mask_not_null, new_cols] = df_new
         df.drop(col, axis=1, inplace=True)
 
-
+        
 def _list_dictionary_to_df(df, col, value_name, var_name):
     """ Some columns are a list of dictionaries. This turns them into a new dataframe of rows."""
     df = df.loc[df[col].notnull(), ['id', col]]
@@ -53,9 +58,10 @@ def _split_dict_col(df, col):
     return df
 
 
-def _simplify_cols_and_drop(df, col):
+def _simplify_cols_and_drop(df, col, cols=None):
     """ Function to merge similar columns together and drop original columns. """
-    cols = df.columns[df.columns.str.contains(col)]
+    if cols is None:
+        cols = df.columns[df.columns.str.contains(col)]
     df[col] = df.lookup(df.index, df[cols].notnull().idxmax(axis=1))
     df.drop(cols, axis=1, inplace=True)
     return df
@@ -117,6 +123,9 @@ def read_event(path_or_buf, related_event_df=True, shot_freeze_frame_df=True, ta
     
     # read as dataframe
     df = pd.read_json(path_or_buf, encoding='utf-8')
+    if df.empty:
+        print(f'Skipping {path_or_buf}: empty json')
+        return
     
     # timestamp defaults to today's date so store as integers in seperate columns
     df['timestamp_minute'] = df.timestamp.dt.minute
@@ -163,7 +172,9 @@ def read_event(path_or_buf, related_event_df=True, shot_freeze_frame_df=True, ta
     df = _simplify_cols_and_drop(df, 'body_part_id')
     df = _simplify_cols_and_drop(df, 'body_part_name')
     df = _simplify_cols_and_drop(df, 'aerial_won')
-
+    df = _simplify_cols_and_drop(df, 'end_x', ['pass_end_x', 'carry_end_x', 'shot_end_x', 'goalkeeper_end_x'])    
+    df = _simplify_cols_and_drop(df, 'end_y', ['pass_end_y', 'carry_end_y', 'shot_end_y', 'goalkeeper_end_y'])    
+    
     # create a related events dataframe
     if related_event_df:
         df_related_event = _list_dictionary_to_df(df, col='related_events',
@@ -205,6 +216,9 @@ def read_event(path_or_buf, related_event_df=True, shot_freeze_frame_df=True, ta
     
     # drop columns stored as a separate table
     df.drop(['related_events', 'shot_freeze_frame', 'tactics_lineup'], axis=1, inplace=True)
+    
+    # rename end location
+    df.rename({'shot_end_z': 'end_z'}, axis=1, inplace=True)
            
     # reorder columns so some of the most used ones are first
     cols = ['match_id', 'id', 'index', 'period', 'timestamp_minute', 'timestamp_second', 
@@ -212,9 +226,8 @@ def read_event(path_or_buf, related_event_df=True, shot_freeze_frame_df=True, ta
             'outcome_id', 'outcome_name',  'play_pattern_id', 'play_pattern_name',
             'possession_team_id', 'possession',  'possession_team_name', 'team_id', 'team_name',
             'player_id', 'player_name', 'position_id',
-            'position_name', 'duration', 'x', 'y', 'pass_end_x', 'pass_end_y', 'carry_end_x',
-            'carry_end_y', 'shot_end_x', 'shot_end_y', 'shot_end_z',
-            'goalkeeper_end_x', 'goalkeeper_end_y', 'body_part_id', 'body_part_name']
+            'position_name', 'duration', 'x', 'y', 'end_x', 'end_y', 'end_z',
+            'body_part_id', 'body_part_name']
     other_cols = df.columns[~df.columns.isin(cols)]
     cols.extend(other_cols)
     df = df[cols].copy()
@@ -266,12 +279,16 @@ def read_match(path_or_buf, warn=True):
         warnings.warn(statsbomb_warning)
         
     df_match = pd.read_json(path_or_buf, convert_dates=['match_date', 'last_updated'])
+    if df_match.empty:
+        print(f'Skipping {path_or_buf}: empty json')
+        return
     
     # loop through the columns that are still dictionary columns and add them as seperate cols to the datafram
     dictionary_columns = ['competition', 'season', 'home_team', 'away_team', 'metadata', 'competition_stage',
                           'stadium', 'referee']
     for col in dictionary_columns:
-        df_match = _split_dict_col(df_match, col)
+        if col in df_match.columns:
+            df_match = _split_dict_col(df_match, col)
         
     # convert kickoff to datetime - date + kickoff time
     df_match['kick_off'] = pd.to_datetime(df_match.match_date.astype(str) + ' ' + df_match.kick_off)
@@ -336,6 +353,9 @@ def read_competition(path_or_buf, warn=True):
         warnings.warn(statsbomb_warning)
         
     df_competition = pd.read_json(path_or_buf, convert_dates=['match_updated', 'match_available'])
+    if df_competition.empty:
+        print(f'Skipping {path_or_buf}: empty json')
+        return
     df_competition.sort_values(['competition_id', 'season_id'], inplace=True)
     df_competition.reset_index(drop=True, inplace=True)
     return df_competition
@@ -382,6 +402,9 @@ def read_lineup(path_or_buf, warn=True):
         warnings.warn(statsbomb_warning)
         
     df_lineup = pd.read_json(path_or_buf)
+    if df_lineup.empty:
+        print(f'Skipping {path_or_buf}: empty json')
+        return
     df_lineup['match_id'] = os.path.basename(path_or_buf[:-5])
     # each line has a column named player that contains a list of dictionaries
     # we split into seperate columns and then create a new row for each player using melt
